import typer

CLI_OPTIONS = {
    'file_type_code'  : typer.Option(..., help='Код типа файла'),
    'file_type_name'  : typer.Option(..., help='Наименование типа файла'),
    'file_info_name'  : typer.Option('Протокол осмотра', help='Наименование файла'),
    'create_user_id'  : typer.Option(1, help='Идентификатор пользователя, прикрепившего файл'),
    'create_user_name': typer.Option('Администратор', help='ФИО пользователя, прикрепившего файл'),
    'prefix'          : typer.Option('', help='Префикс имени файла'),
    'suffix'          : typer.Option('', help='Суффикс имени файла'),
    'recursive'       : typer.Option(False, help='Рекурсивный поиск'),
    'dry_run'         : typer.Option(False, help='Тестовый запуск'),
    'mis_db_server'   : typer.Option(..., envvar='MIS_DB_SERVER', help='Адрес сервера МИС'),
    'mis_db_port'     : typer.Option(1433, envvar='MIS_DB_PORT', help='Порт сервера МИС'),
    'mis_db_name'     : typer.Option(..., envvar='MIS_DB_NAME', help='Имя базы данных'),
    'mis_db_username' : typer.Option('sa', envvar='MIS_DB_USERNAME', help='Имя пользователя для подключения к БД МИС'),
    'mis_db_password' : typer.Option(..., envvar='MIS_DB_PASSWORD',
                                     help='Пароль пользователя для подключения к БД МИС'),
    'mis_file_path'   : typer.Option(..., envvar='MIS_FILE_PATH', help='Путь до хранилища прикреплённых файлов МИС'),
    'log_path'        : typer.Option(None, envvar='PMA_LOG_PATH', help='Путь для хранения журнала приложения'),
}



import pymssql


class Database(object):
    def __init__(self, server, port, name, username, password):
        self.connection = pymssql.connect(host=server, port=port, user=username, password=password, database=name)

    def execute_query(self, sql: str, params: tuple = ..., as_dict=False):
        cursor = self.connection.cursor(as_dict=as_dict)
        if params is ...:
            cursor.execute(sql)
        else:
            cursor.execute(sql, params)

        return cursor

    def select_all(self, sql: str, params: tuple = ..., as_dict=False):
        cursor = self.execute_query(sql, params, as_dict)
        return cursor.fetchall()

    def select_one(self, sql: str, params: tuple = ..., as_dict=False):
        cursor = self.execute_query(sql, params, as_dict)
        return cursor.fetchone()



GET_ATTACHMENT_PATH = "select ValueStr from x_UserSettings where Property = N'Путь к хранилищу прикреплённых файлов'"
GET_FILETYPE_ID = "select FileTypeID from atf_FileType where Code = %s"
CREATE_FILETYPE = "insert into atf_FileType (Code, Name) values (%s, %s)d"
GET_DOCUMENT_GUID = "select GUID from x_DocTypeDef where HeadTable = 'hlt_MKAB'"
GET_FILEINFO = """
select top 1 FileInfoID
from atf_FileInfo
where rf_FileTypeID = %d
  and rf_DescTypeGuid = %s
  and DescGuid = %s
  and cast(DateDoc as date) = cast(getdate() as date)
"""
CREATE_FILEINFO = "insert into atf_FileInfo (rf_FileTypeID, rf_DescTypeGuid, DescGuid) values (%d, %s, %s)"
CREATE_ATTACHMENT = """
insert into atf_FileAttachment (rf_CreateUserID, CreateUserName, rf_FileInfoID, Path, Description)
values (%d, %s, %d, %s, %s)
"""
GET_MKAB = "select * from hlt_MKAB where NUM = %s"



import re
import shutil
import uuid
from pathlib import Path

import pkg_resources
import typer
from loguru import logger

from .database import Database
from .options import CLI_OPTIONS
from .sql import *

try:
    __version = pkg_resources.get_distribution("pm-attacher").version
except pkg_resources.DistributionNotFound:
    __version = 'dev'

app = typer.Typer()


@app.command()
def version():
    try:
        _version = pkg_resources.get_distribution("pm-attacher").version
    except pkg_resources.DistributionNotFound:
        _version = 'dev'

    typer.echo(f'PM Attacher, version: {__version}')


@app.command()
def main(
        watch_dir: Path,
        file_type_code: str = CLI_OPTIONS['file_type_code'],
        file_type_name: str = CLI_OPTIONS['file_type_name'],
        file_info_name: str = CLI_OPTIONS['file_info_name'],
        create_user_id: int = CLI_OPTIONS['create_user_id'],
        create_user_name: str = CLI_OPTIONS['create_user_name'],
        prefix: str = CLI_OPTIONS['prefix'],
        suffix: str = CLI_OPTIONS['suffix'],
        recursive: bool = CLI_OPTIONS['recursive'],
        dry_run: bool = CLI_OPTIONS['dry_run'],
        mis_db_server: str = CLI_OPTIONS['mis_db_server'],
        mis_db_port: int = CLI_OPTIONS['mis_db_port'],
        mis_db_name: str = CLI_OPTIONS['mis_db_name'],
        mis_db_username: str = CLI_OPTIONS['mis_db_username'],
        mis_db_password: str = CLI_OPTIONS['mis_db_password'],
        mis_file_path: Path = CLI_OPTIONS['mis_file_path'],
        log_path: Path = CLI_OPTIONS['log_path']
):
    if log_path is not None:
        logger.add(
            log_path / 'debug.log', format='{time:YYYY-MM-DD HH:mm:ss.SSS} | {level: <8} | {message}',
            level='DEBUG', rotation='1 MB', retention='3 months', compression='tar.gz'
        )

    glob_pattern = f'{prefix}*{suffix}.*'
    if recursive:
        glob_pattern = '**/' + glob_pattern

    # Инициализируем подключение к БД
    db = Database(mis_db_server, mis_db_port, mis_db_name, mis_db_username, mis_db_password)

    if dry_run:
        logger.warning('программа запущена о режиме, не вносящем изменения!')

    for filename in Path(watch_dir).glob(glob_pattern):
        match = re.match(r'^%s(\d+)%s' % (prefix, suffix), filename.name)
        if not match:
            logger.debug(f'в имени файла {filename.absolute()} НЕ обнаружен идентификатор')
            continue

        mkab_num = match.group(1)
        logger.debug(f'в имени файла {filename.absolute()} обнаружен идентификатор: {mkab_num}')

        # Получение МКАБ по номеру карты
        try:
            mkab = db.select_all(GET_MKAB, (mkab_num,), as_dict=True)[0]
            logger.debug(f'МКАБ {mkab_num} найден')
        except IndexError:
            logger.error(f'МКАБ {mkab_num} не найден')
            continue

        # Копирование файла в директорию назначения (mis-files)
        new_filename = mis_file_path / f'{uuid.uuid4()}{filename.suffix}'
        if not dry_run:
            shutil.copy(filename, new_filename)
        logger.debug(f'файл {filename.absolute()} скопирован в {new_filename.absolute()}')

        # Находим в настройках МИС, где должны лежать файлы для всеобщего обозрения
        try:
            path = db.select_one(GET_ATTACHMENT_PATH)[0]
        except (TypeError, KeyError):
            logger.error('Не найден параметр "Путь к хранилищу прикреплённых файлов" в таблице x_UserSettings')
            raise RuntimeError

        # Находим необходимый тип прикреплённых файлов
        try:
            file_type_id = db.select_one(GET_FILETYPE_ID, (file_type_code,))[0]
        except (TypeError, KeyError):
            cursor = db.execute_query(CREATE_FILETYPE, (file_type_code, file_type_name))
            file_type_id = cursor.lastrowid

        # Находим идентификатор типа документа МКАБ
        desc_type_guid = db.select_one(GET_DOCUMENT_GUID)[0]

        # Получение записи с информацией о группе вложений
        try:
            # Ищем уже имеющуюся информацию о файле на текущий день
            file_info_id = db.select_one(GET_FILEINFO, (file_type_id, desc_type_guid, mkab['UGUID']))[0]
            logger.debug(f'Используем имеющуюся информацию о файле (FileInfoID={file_info_id})')
        except (TypeError, KeyError):
            # Если не нашли - добавляем информацию о файле
            cursor = db.execute_query(CREATE_FILEINFO, (file_type_id, desc_type_guid, mkab['UGUID']))
            file_info_id = cursor.lastrowid
            logger.debug(f'Используем вновь созданную информацию о файле (FileInfoID={file_info_id})')

        # Добавляем информацию о вложении
        db.execute_query(CREATE_ATTACHMENT, (
            create_user_id,
            create_user_name,
            file_info_id,
            '\\'.join((path, new_filename.name)),
            file_info_name
        ))

        # Удаляем файл и коммитим изменения
        if not dry_run:
            filename.unlink()
            logger.debug(f'файл {filename.absolute()} удалён')
            db.connection.commit()


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pm_attacher']

package_data = \
{'': ['*']}

install_requires = \
['loguru>=0.5.3,<0.6.0', 'pymssql>=2.2.2,<3.0.0', 'typer>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['pm-attacher = pm_attacher.main:app']}

setup_kwargs = {
    'name': 'pm-attacher',
    'version': '0.2.0',
    'description': "Attach files to a patient's medical record (TrustMed)",
    'long_description': '![python-version](https://img.shields.io/badge/python-3.7+-blue.svg)\n\n# Прикрепление файлов к МКАБ\n\nУтилита предназначена для поиска и прикрепления файлов к медицинским картам амбулаторных больных (МКАБ), основываясь на имени файла, в котором должен быть номер МКАБ.\n\n## Установка\n\n```bash\npip install pm-attacher\n```\n\n## Настройка\n\nУправление параметрами утилиты производится либо через переменные окружения, либо через ключи запуска, причём приоритет имеют последние.\n\n| Ключ запуска         | Переменная окружения | Обязательно? | Значение по умолчанию | Описание                                       |\n|----------------------|----------------------|:------------:|-----------------------|------------------------------------------------|\n| `--file-type-code`   |                      |      Да      |                       | Код типа файла                                 | \n| `--file-type-name`   |                      |      Да      |                       | Наименование типа файла                        | \n| `--file-info-name`   |                      |     Нет      | `Протокол осмотра`    | Наименование файла                             | \n| `--create-user-id`   |                      |     Нет      | `1`                   | Идентификатор пользователя, прикрепившего файл | \n| `--create-user-name` |                      |     Нет      | `Администратор`       | ФИО пользователя, прикрепившего файл           | \n| `--prefix`           |                      |     Нет      |                       | Префикс имени файла                            |         \n| `--suffix`           |                      |     Нет      |                       | Суффикс имени файла                            |         \n| `--recursive`        |                      |     Нет      | `False`               | Рекурсивный поиск                              |      \n| `--dry-run`          |                      |     Нет      | `False`               | Тестовый запуск, изменения не сохраняются      |        \n| `--mis-db-server`    | `MIS_DB_SERVER`      |      Да      |                       | Адрес сервера МИС                              |  \n| `--mis-db-port`      | `MIS_DB_PORT`        |     Нет      | `1433`                | Порт сервера МИС                               |    \n| `--mis-db-name`      | `MIS_DB_NAME`        |      Да      |                       | Наименование базы данных                       |    \n| `--mis-db-username`  | `MIS_DB_USERNAME`    |     Нет      | `sa`                  | Имя пользователя для подключения к БД МИС      |\n| `--mis-db-password`  | `MIS_DB_PASSWORD`    |      Да      |                       | Пароль пользователя для подключения к БД МИС   |\n| `--mis-file-path`    | `MIS_FILE_PATH`      |      Да      |                       | Путь до хранилища прикреплённых файлов МИС     |  \n| `--log-path`         | `PMA_LOG_PATH`       |     Нет      |                       | Путь для хранения журнала приложения           |         \n\nКлючи `--create-user-id` и `--create-user-name` должны соответствовать полям `UserID` и `FIO` таблицы `x_User`.\n\nКлючи `--file-type-code` и `--file-type-name` должны соответствовать полям `Code` и `Name` таблицы `atf_FileType`. Если совпадение по коду не будет найдено, то программа создаст тип прикрепляемого файла с указанными параметрами. **Внимание!** Поиск типа производится по полю `Code` и используется первое найденное вхождение.\n\nКлючи `--recursive` и `--dry-run` являются флагами и им **не передаются** значения.\n\nПри использовании ключа `--log-path` в указанной директории будет формироваться журнал `debug.log` с ротацией каждые 1 Мб и очисткой данных через 3 месяца.\n\nПолучить справку по описанным выше параметрам из командной строки можно запустив утилиту с ключом `--help`:\n\n```bash\npm-attacher --help\n```\n\n## Запуск\n\nПри запуске необходимо определить обязательные параметры (таблица выше) и указать путь, по которому будет производиться обработка файлов:\n\n```bash\npm-attacher [OPTIONS] WATCH_DIR\n```\n',
    'author': 'Vladislav Chmelyuk',
    'author_email': 'neimp@yandex.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
